:: XcopyWithVSS.bat - This script will copy all the files in a folder, and underlying subfolders into a new folder using Xcopy and Volume Shadowing
:: Usage: XCopyWithVSS.bat "<inputfilespec>" "<outputFolder>" "<OS-type>" Note the output folder should NOT have a trailing /
:: Valid OS-types are: XP, 2003, 32 and 64
:: There is a package of file dependencies that accompany this script including vshadow distributions
:: 15-Feb-2010 - Portions adapted from tech posting on the Internet - MDR - Dr.Backup
:: 11/25/2013 - Added some basic error handling

SETLOCAL

:: Customization parameters which can be adjusted
SET FREE_DRIVE_LETTER=Q:
SET XCOPY_OPTIONS=/E /C /H /O /Y /I
:: Note: Add /M to options string in order to perform incremental copy using archive flag.
:: Note: Remove /E to exclude subfolders from copy operation

:: Do not change anything below this line


:: The version of VSHADOW is different on each OS. Specify it as parameter 3
:: Valid values are XP, 2003, 32 or 64 - default in build will be 32 (Vista 32 or Windows 7 -32bit)
SET OS=%3

:: If this is a callback from VSHADOW, then skip parameter setup
IF NOT "%CALLBACK_SCRIPT%"=="" GOTO :IS_CALLBACK

:: Extract the drive letter for use for the VSS Shadow, the relative path to append to the FREE_DRIVE_LETTER
:: and the destination folder to put the split files into. These parameters are all coming from the command line
SET SOURCE_DRIVE_LETTER=%~d1
SET SOURCE_RELATIVE_PATH=%~pnx1
SET DESTINATION_PATH=%2

:: We will be telling VSHADOW to do a recursive callback to this batch job, specified as parameter %0 in command line
SET CALLBACK_SCRIPT=%~dpnx0

:: Tell VSHADOW where to write the Volume shadow filespec after created
:: We will execute this batch script after the callback - Hardcoded the script name
SET TEMP_GENERATED_SCRIPT=GeneratedVarsTempScript.cmd

:: Lets actually create the volume shadow set now. VSHADOW will call us back when done and load definitions
:: into the TEMP_GENERATED_SCRIPT command file.
"%~dp0VSHADOW%OS%.EXE" -script="%TEMP_GENERATED_SCRIPT%" -exec="%CALLBACK_SCRIPT%" %SOURCE_DRIVE_LETTER%

:: By time we get here, the callback has occured and we are done with the temporarily generated script to give us
:: the shadow file spec. Get rid of temp file
DEL /F %TEMP_GENERATED_SCRIPT%

:: We are done.
GOTO :EOF

:: Here is the magic. We are called and wind up here while the Shadow volume still exists and is mapped to a drive
:: Lets do our testing and Gsplit if file big enough

:IS_CALLBACK
SETLOCAL

:: Lets call the script created by VSHADOW to let us know the device name of the shadow volume so we can map it to a drive
:: The script should set the SHADOW_DEVICE_1 env variable
CALL %TEMP_GENERATED_SCRIPT%

:: To use the shadow volume freely, lets assign it a drive letter
:: We are using the DOSDEV utility here provided in the package
:: 8/31/2011 - remove the drive before attempting to assign
"%~dp0DOSDEV" -d -r %FREE_DRIVE_LETTER%
"%~dp0DOSDEV" %FREE_DRIVE_LETTER% %SHADOW_DEVICE_1%

:: Now that everything is setup, we can use the drive letter to do an Xcopy
:: Note: Add /M to the options below to perform an incremental copy using archive flag.

:: Delete error file from previous run if present, then initiate copy
:: To delete all files, remove ::: lines below. Careful - configuration errors in destination location could result in unintended deletions of good data!!!
:::DEL /F /Q /S "%DESTINATION_PATH:"=%\*.*"
DEL /F /Q "%DESTINATION_PATH:"=%\XC-##CUSTOMERROR##.TXT"
XCOPY "%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%" %DESTINATION_PATH%  %XCOPY_OPTIONS%

:: Add in some basic error checking here
IF %ERRORLEVEL% EQU 0 GOTO :EOF
:::DEL /F /Q /S "%DESTINATION_PATH:"=%\*.*"
ECHO XCOPYWITHVSSERROR-%FREE_DRIVE_LETTER%%SOURCE_RELATIVE_PATH%(VSSXCopyError) > "%DESTINATION_PATH:"=%\XC-##CUSTOMERROR##.TXT"
:EOF
:: All done here